/* ./RQLEngine/src/main/java/dag/satmining/problem/rql/ast/sql/CacheTupleFetcher.java

   Copyright (C) 2013, 2014 Emmanuel Coquery.

This file is part of SATMiner

SATMiner is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

SATMiner is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with SATMiner; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version. */

package dag.satmining.problem.rql.ast.sql;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public final class CacheTupleFetcher implements TupleFetcher {

	private final Tuple[] _data;
	private final int _size;
	private int _current;
	private final Map<String,Integer> _schema;
	
	public CacheTupleFetcher(TupleFetcher backend) throws SQLException, IOException {
		this._schema = backend.getSchema();
		List<Tuple> data = new ArrayList<Tuple>();
		while(backend.next()) {
			data.add(backend.getTuple());
		}
		_size = data.size();
		_data = data.toArray(new Tuple[_size]);
		reset();
	}
	
	@Override
	public boolean next() throws SQLException, IOException {
		_current++;
		return _current < _size;
	}

	@Override
	public Tuple getTuple() throws SQLException, IOException {
		return _data[_current];
	}

	@Override
	public void reset() throws SQLException, IOException {
		_current = -1;
	}

	@Override
	public Map<String, Integer> getSchema() throws SQLException, IOException {
		return _schema;
	}

}
